/*
 * This file is part of Deliantra, the Roguelike Realtime MMORPG.
 *
 * Copyright (©) 2005,2006,2007,2008,2009,2010,2011,2012 Marc Alexander Lehmann / Robin Redeker / the Deliantra team
 * Copyright (©) 1994-2004 Crossfire Development Team (restored, original file without copyright notice)
 *
 * Deliantra is free software: you can redistribute it and/or modify it under
 * the terms of the Affero GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the Affero GNU General Public License
 * and the GNU General Public License along with this program. If not, see
 * <http://www.gnu.org/licenses/>.
 *
 * The authors can be reached via e-mail to <support@deliantra.net>
 */

/* peterm@langmuir.eecs.berkeley.edu:  this function generates a random
snake-type maze.

input:  xsize, ysize;
output:  a char** array with # and . for closed and open respectively.

a char value of 0 represents a blank space:  a '#' is
a wall.

*/

#include <global.h>
#include <rmg.h>
#include "rproto.h"

void
make_snake_layout (layout &maze, int options)
{
  int i, j;

  maze.clear ();
  maze.border ();

  int xsize = maze.w;
  int ysize = maze.h;

  /* Bail out if the size is too small to make a snake. */
  if (xsize < 8 || ysize < 8)
    return;

  /* decide snake orientation--vertical or horizontal , and
     make the walls and place the doors. */

  if (rmg_rndm (2))
    {                           /* vertical orientation */
      int n_walls = rmg_rndm (xsize - 5) / 3 + 1;
      int spacing = xsize / (n_walls + 1);
      int orientation = 1;

      for (i = spacing; i < xsize - 3; i += spacing)
        {
          if (orientation)
            {
              for (j = 1; j < ysize - 2; j++)
                maze[i][j] = '#';

              maze[i][j] = 'D';
            }
          else
            {
              for (j = 2; j < ysize; j++)
                maze[i][j] = '#';

              maze[i][1] = 'D';
            }

          orientation ^= 1;     /* toggle the value of orientation */
        }
    }
  else
    {                           /* horizontal orientation */
      int n_walls = rmg_rndm (ysize - 5) / 3 + 1;
      int spacing = ysize / (n_walls + 1);
      int orientation = 1;

      for (i = spacing; i < ysize - 3; i += spacing)
        {
          if (orientation)
            {
              for (j = 1; j < xsize - 2; j++)
                maze[j][i] = '#';

              maze[j][i] = 'D';
            }
          else
            {
              for (j = 2; j < xsize; j++)
                maze[j][i] = '#';

              maze[1][i] = 'D';
            }

          orientation ^= 1;     /* toggle the value of orientation */
        }
    }

  /* place the exit up/down */
  if (rmg_rndm (2))
    {
      maze[1][1] = '<';
      maze[xsize - 2][ysize - 2] = '>';
    }
  else
    {
      maze[1][1] = '>';
      maze[xsize - 2][ysize - 2] = '<';
    }
}

